#include "testApp.h"

//--------------------------------------------------------------
void testApp::setup(){	
	
	ofBackground(0,0,0);
	
	ofSetVerticalSync(true);
	ofSetFrameRate(60);
	
	// load in sounds:
	sufjan.loadSound("sounds/song.mp3");
	
	// the fft needs to be smoothed out, so we create an array of floats
	// for that purpose:
	fftSmoothed = new float[8192];
	for (int i = 0; i < 8192; i++){
		fftSmoothed[i] = 0;
	}
	nBandsToGet = 128;
	sufjan.play();
	
	for (int i = 0; i < 350; i++){
		particle myParticle;
		
		float x = 500 + 100 * cos ( (i / 200.0) * TWO_PI);
		float y = 500 + 100 * sin ( (i / 200.0) * TWO_PI);
		
		myParticle.setInitialCondition(x,y-100 ,0,0);
		myParticle.damping = 0.45f;
		particles.push_back(myParticle);
	}
	
	// change this to
	// for (int i = 0; i < 10; i++){
	// to see a chain
	
	for (int i = 0; i < particles.size(); i++){
		spring mySpring;
		mySpring.distance		= 3;
		mySpring.springiness	= 0.4f;
		mySpring.particleA = & (particles[i  ]);
		mySpring.particleB = & (particles[(i+1) % particles.size()]);
		springs.push_back(mySpring);
		}
	
	particle myContourParticle;
	myContourParticle.setInitialCondition(ofGetWidth()/2, ofGetHeight()/2, 0, 0);
	contourParticles.push_back(myContourParticle);
	
	
		
	
}

//--------------------------------------------------------------
void testApp::update(){
	
	// update the sound playing system:
	ofSoundUpdate();	
	sufjan.setVolume(2);
	
	// (5) grab the fft, and put in into a "smoothed" array,
	//		by taking maximums, as peaks and then smoothing downward
	float * val = ofSoundGetSpectrum(nBandsToGet);		// request 128 values for fft
	for (int i = 0;i < nBandsToGet; i++){
		
		// let the smoothed value sink to zero:
		fftSmoothed[i] *= 0.96f;
		
		// take the max, either the smoothed or the incoming:
		if (fftSmoothed[i] < val[i]) fftSmoothed[i] = val[i];
		
	}
	
	float sinOfTime				= sin( ofGetElapsedTimef() );
	float sinOfTimeMapped		= ofMap( sinOfTime, -1, 1, 0, 600)*1.76;
	float sinOfTimeMapped2		= ofMap( sinOfTime, -1, 1, 0, 800)*10.12;
	
	// on every frame 
	// we reset the forces
	// add in any forces on the particle
	// perfom damping and
	// then update
	
	for (int i = 0; i < particles.size(); i++){
		particles[i].resetForce();
	}
	
	for (int i = 0; i < particles.size(); i++){
		
		particles[i].addRepulsionForce(mouseX, mouseY, 200, 0.7f);
		
		for (int j = 0; j < i; j++){
			particles[i].addRepulsionForce(particles[j], 50, 0.1);
			if (fftSmoothed[i] > 0.01) {
			contourParticles[0].addRepulsionForce(particles[j], sinOfTimeMapped, 0.6f);
			}else{
			contourParticles[0].addAttractionForce(particles[j], sinOfTimeMapped2, 0.1f);
				}
			if (fftSmoothed[j] > 0.01) {
				contourParticles[0].addClockwiseForce(particles[i], sinOfTimeMapped, 0.6f);
			}else{
				contourParticles[0].addCounterClockwiseForce(particles[i], sinOfTimeMapped2, 0.1f);
			}

			}
		}
	
	
	for (int i = 0; i < springs.size(); i++){
		springs[i].update();
	}
	
	
	for (int i = 0; i < particles.size(); i++){
		particles[i].bounceOffWalls();
		particles[i].addDampingForce();
		particles[i].update();
	}
	
	
}

//--------------------------------------------------------------
void testApp::draw(){
	
	ofEnableAlphaBlending();
	
	//for (int i = 0; i < contourParticles.size(); i++) {
//		contourParticles[i].draw();
//	}
	
	for (int i = 0; i < particles.size(); i++){
		ofSetColor(100,i,50,50);
		particles[i].draw();
	}
	
	for (int i = 0; i < springs.size(); i++){
		ofSetColor(50,100,i,50);
		springs[i].draw();
	}
	
	for (int i = 0; i < particles.size(); i++){
		for (int j = i-1; j >= 0; j--){
			if ((abs(particles[i].pos.x - particles[j].pos.x) > 66 && abs(particles[i].pos.y - particles[j].pos.y) > 66) && (abs(particles[i].pos.x - particles[j].pos.x) < 100 && abs(particles[i].pos.y - particles[j].pos.y) < 100)){
				ofSetColor(i,50,100,50);
				ofLine(particles[i].pos.x, particles[i].pos.y, particles[j].pos.x, particles[j].pos.y);
			}
		}
	}	
	
	
	for (int i = 0; i < particles.size(); i++){
		for (int j = i-1; j >= 0; j--){
			if ((abs(particles[i].pos.x - particles[j].pos.x) > 33 && abs(particles[i].pos.y - particles[j].pos.y) > 33) && (abs(particles[i].pos.x - particles[j].pos.x) < 65 && abs(particles[i].pos.y - particles[j].pos.y) < 65)){
				ofSetColor(i,100,50,50);
				ofLine(particles[i].pos.x, particles[i].pos.y, particles[j].pos.x, particles[j].pos.y);
			}
		}
	}

	
	
	for (int i = 0; i < particles.size(); i++){
		for (int j = i-1; j >= 0; j--){
			if ((abs(particles[i].pos.x - particles[j].pos.x) > 0 && abs(particles[i].pos.y - particles[j].pos.y) > 0) && (abs(particles[i].pos.x - particles[j].pos.x) < 32 && abs(particles[i].pos.y - particles[j].pos.y) < 32)){
				ofSetColor(i,100,150,50);
				ofLine(particles[i].pos.x, particles[i].pos.y, particles[j].pos.x, particles[j].pos.y);
			}
		}
	}	
	
	
	for (int i = 0; i < particles.size(); i++){
		for (int j = i-1; j >= 0; j--){
			if ((abs(particles[i].pos.x - particles[j].pos.x) > 300 && abs(particles[i].pos.y - particles[j].pos.y) > 300) && (abs(particles[i].pos.x - particles[j].pos.x) < 100 && abs(particles[i].pos.y - particles[j].pos.y) < 100)){
				ofSetColor(i,150,100,50);
				ofLine(particles[i].pos.x, particles[i].pos.y, particles[j].pos.x, particles[j].pos.y);
			}
		}
	}	

	
}

//--------------------------------------------------------------
void testApp::keyPressed  (int key){ 
	
	switch (key){
			
		case ' ':
			// reposition everything: 
			for (int i = 0; i < particles.size(); i++){
				particles[i].setInitialCondition(ofRandom(0,ofGetWidth()),ofRandom(0,ofGetHeight()),0,0);
			}
			break;
	}
	
	
}

//--------------------------------------------------------------
void testApp::keyReleased  (int key){ 
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	particles[0].pos.set(mouseX, mouseY);
	/*particles.erase(particles.begin());
	particle myParticle;
	myParticle.setInitialCondition(x,y,0,0);
	particles.push_back(myParticle);*/
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	particles[0].bFixed = true;
}

//--------------------------------------------------------------
void testApp::mouseReleased(){
	
	particles[0].bFixed = false;
}

